RSpec.shared_examples "optional reference" do |name|
  describe "##{name}" do
    it 'not include in request' do
      expect(find_entity.send(name)).to be_nil
    end

    it 'is include in request' do
      expect(find_entity("with_#{name}").send(name)).to be_a "Easy::Resources::Redmine::#{name.to_s.classify}".constantize
    end
  end
end

RSpec.describe Easy::Resources::Redmine::EasyWebApplication do
  include_context "redmine active resource shares" do
    let(:entity_name) { 'easy_web_application' }
  end

  it 'get ewa' do
    expect(find_entity.attributes).to include(*%w[id url git_repository version application_type created_at])
    expect(find_entity.git_branch).to be_nil
    expect(find_entity.name).not_to be_empty
  end

  it 'ewa has easy_server' do
    expect(find_entity.easy_server).to be_a Easy::Resources::Redmine::EasyServer
  end

  include_examples "optional reference", :easy_crm_case
  include_examples "optional reference", :easy_contact

  describe ".get_all" do
    it "with block" do
      stub_xml(path: "easy_web_applications.xml", source_file: "index", response_headers: { 'Total' => '4' })
      stub_xml(path: "easy_web_applications.xml?page=2", source_file: "index")
      expect(described_class.get_all.count).to eq 4
    end
  end

  it "create EWA" do
    stub_xml(method: :post, path: "easy_web_applications.xml", request_body: { easy_web_application: { url: "xxx" } }.to_xml, response_body: "")
    ewa = described_class.new(url: "xxx")
    expect { ewa.save }.not_to raise_error
  end

  describe "easy_web_application_activity" do
    subject { described_class.new(url: "awesome.ewa") }

    it "update EWA with activity" do
      req = ActiveResource::Request.new(:post, "/easy_web_applications.xml", file_fixture("easy_web_application/with_ewaa.xml").read, { "Content-Type" => "application/xml", "X-Redmine-API-Key" => api_key })
      res = ActiveResource::Response.new("", 201, {})
      ActiveResource::HttpMock.respond_to({ req => res })
      subject.easy_web_application_activities_attributes = { activities_count_in_day: 13 }
      subject.save
    end
  end

  describe "application_moves_on_request?" do
    subject { find_entity.application_moves_on_request? }
    it "should be true" do
      is_expected.to be_truthy
    end
  end

  describe "application_updates_on_request?" do
    subject { find_entity.application_updates_on_request? }
    it "should be false" do
      is_expected.to be_falsey
    end
  end
end
