class DummyResource < Easy::Resource
  self.site = "easyredmine.com"
  self.format = :json

end
RSpec.describe Easy::Resource do
  subject { DummyResource.new({ id: 1 }, true) }
  let(:value) { { att: 1 } }

  it "#becomes" do
    target_class = Easy::Resources::Redmine::Issue
    expect(subject.becomes(target_class)).to be_a(target_class).and have_attributes id: 1
  end

  describe "#update_column" do
    it "json" do
      expect(subject).to receive(:patch).with nil, {}, value.to_json
      subject.update_column :att, 1
    end

    it "xml" do
      allow(DummyResource).to receive(:format).and_return ActiveResource::Formats::XmlFormat
      expect(subject).to receive(:patch).with nil, {}, value.to_xml(root: "dummy-resource")
      subject.update_column :att, 1
    end
  end

  describe "#patch" do
    let(:connection) { double "ActiveResource::Connection" }
    before(:each) { allow(subject).to receive(:connection).and_return connection }

    it "update columns" do
      expect(connection).to receive(:patch).with "easyredmine.com/dummy_resources/1.json", value.to_json, {}
      subject.update_columns value
    end

    it "custom url" do
      expect(connection).to receive(:patch).with "easyredmine.com/dummy_resources/1/deployed.json", "", {}
      subject.patch "deployed"
    end
  end
end
