RSpec.shared_context "active resource shares", shared_context: :metadata do
  let(:site) { 'https://test.dummy' }
  let(:api_key) { 'secret' }

  let(:entity_name) { "" }

  before(:each) do
    ActiveResource::HttpMock.reset!
  end

  def find_entity(source_file = nil)
    stub_xml source_file: (source_file || "show"), path: "#{entity_name.pluralize}/1.xml"

    described_class.find 1
  end

  # @param [Hash] options @see #stub_easy_resource_request
  def stub_json(**options)
    options[:format] = "json"
    stub_resource_request(**options)
  end

  # @param [Hash] options @see #stub_easy_resource_request
  def stub_xml(**options)
    options[:format] = "xml"
    stub_resource_request(**options)
  end

  # @param [String] format XML or JSON
  # @param [String] path relative path in uri
  # @param [Hash] options Request&response options
  # @option options [String] :method (:get) HTTP method if request
  # @option options [Integer] :status (200) response status code
  # @option options [String] :source_file path for fixture
  # @option options [String] :response_body
  # @option options [String] :request_body
  # @option options [String] :request_headers ({ "X-Redmine-API-Key" => some key })
  # @option options [String] :response_headers ({})
  # @return [ActiveResource::HttpMock::Responder]
  def stub_resource_request(format:, path:, **options)
    response_body = options[:response_body]
    response_body ||= file_fixture("#{entity_name}/#{options[:source_file]}.#{format}").read if options[:source_file]
    raise ArgumentError, "response_body must be given" if response_body.nil?

    method = options[:method] || :get
    r_headers = options[:request_headers] || {}
    if %i[get delete].include?(method.to_sym)
      r_headers["Accept"] = "application/#{format}"
    else
      r_headers["Content-Type"] = "application/#{format}"
    end

    request = ActiveResource::Request.new(method, "/#{path}", options[:request_body], r_headers)
    response = ActiveResource::Response.new(response_body.to_s, options[:status] || 200, options[:response_headers] || {})
    ActiveResource::HttpMock.respond_to({ request => response }, false)
  end
end
